@@ -1,3 +1,4 @@
+#define _XOPEN_SOURCE
 #include <assert.h>
 #include <ctype.h>
 #include <stdbool.h>
@@ -16,7 +17,8 @@ static int parse_section(struct parser *p) {
 	uint32_t ch;
 	while ((ch = parser_getch(p)) != UTF8_INVALID) {
 		if (ch < 0x80 && isdigit(ch)) {
-			assert(str_append_ch(section, ch) != -1);
+			int ret = str_append_ch(section, ch);
+			assert(ret != -1);
 		} else if (ch == ')') {
 			if (!section->str) {
 				break;
@@ -66,13 +68,26 @@ static void parse_preamble(struct parser *p) {
 	int section = -1;
 	uint32_t ch;
 	char date[256];
-	time_t now;
-	time(&now);
-	struct tm *now_tm = localtime(&now);
-	strftime(date, sizeof(date), "%F", now_tm);
+	char *source_date_epoch = getenv("SOURCE_DATE_EPOCH");
+	if (source_date_epoch != NULL) {
+		struct tm source_date_epoch_tm;
+		char *ret = strptime(source_date_epoch, "%s", &source_date_epoch_tm);
+		if (ret == NULL || *ret != '\0') {
+			fprintf(stderr,
+					"Error: $SOURCE_DATE_EPOCH is set but malformed.\n");
+			exit(1);
+		}
+		strftime(date, sizeof(date), "%F", &source_date_epoch_tm);
+	} else {
+		time_t now;
+		time(&now);
+		struct tm *now_tm = localtime(&now);
+		strftime(date, sizeof(date), "%F", now_tm);
+	}
 	while ((ch = parser_getch(p)) != UTF8_INVALID) {
 		if ((ch < 0x80 && isalnum(ch)) || ch == '_' || ch == '-' || ch == '.') {
-			assert(str_append_ch(name, ch) != -1);
+			int ret = str_append_ch(name, ch);
+			assert(ret != -1);
 		} else if (ch == '(') {
 			section = parse_section(p);
 		} else if (ch == '"') {
@@ -472,20 +487,26 @@ static void parse_table(struct parser *p, uint32_t style) {
 			parser_fatal(p, "Expected one of '[', '-', ']', or ' '");
 			break;
 		}
-		if ((ch = parser_getch(p)) != ' ') {
-			parser_fatal(p, "Expected ' '");
-			break;
-		}
-		// Read out remainder of the text
 		curcell->contents = str_create();
-		while ((ch = parser_getch(p)) != UTF8_INVALID) {
-			switch (ch) {
-			case '\n':
-				goto commit_cell;
-			default:
-				assert(str_append_ch(curcell->contents, ch) != -1);
-				break;
+		switch (ch = parser_getch(p)) {
+		case ' ':
+			// Read out remainder of the text
+			while ((ch = parser_getch(p)) != UTF8_INVALID) {
+				switch (ch) {
+				case '\n':
+					goto commit_cell;
+				default:;
+					int ret = str_append_ch(curcell->contents, ch);
+					assert(ret != -1);
+					break;
+				}
 			}
+			break;
+		case '\n':
+			goto commit_cell;
+		default:
+			parser_fatal(p, "Expected ' ' or a newline");
+			break;
 		}
 commit_cell:
 		if (strstr(curcell->contents->str, "T{")
